@@ -10,16 +10,28 @@ import logging # isort:skip
 from glob import glob
 
 from bokeh.command.subcommands.serve import Serve as _BkServe
+from bokeh.command.util import build_single_handler_applications
 
 from ..auth import OAuthProvider
 from ..config import config
 from ..io.rest import REST_PROVIDERS
+from ..io.reload import record_modules, watch
 from ..io.server import INDEX_HTML, get_static_routes
 from ..io.state import state
 
 log = logging.getLogger(__name__)
 
 
+def _cleanup_doc(doc):
+    for callback in doc.session_destroyed_callbacks:
+        try:
+            callback(None)
+        except Exception:
+            pass
+    doc._callbacks[None] = {}
+    doc.destroy(None)
+
+
 def parse_var(s):
     """
     Parse a key, value pair, separated by '='
@@ -109,6 +121,14 @@ class Serve(_BkServe):
             type    = int,
             help    = "The length of the session history to record.",
             default = 0
+        )),
+        ('--warm', dict(
+            action  = 'store_true',
+            help    = "Whether to execute scripts on startup to warm up the server."
+        )),
+        ('--autoreload', dict(
+            action  = 'store_true',
+            help    = "Whether to autoreload source when script changes."
         ))
     )
 
@@ -135,6 +155,13 @@ class Serve(_BkServe):
             else:
                 files.append(f)
 
+        if args.index and not args.index.endswith('.html') and not any(f.endswith(args.index) for f in files):
+            raise ValueError("The --index argument must either specify a jinja2 "
+                             "template with a .html file extension or select one "
+                             "of the applications being served as the default. "
+                             f"The specified application {args.index!r} could "
+                             "not be found.")
+
         # Handle tranquilized functions in the supplied functions
         if args.rest_provider in REST_PROVIDERS:
             pattern = REST_PROVIDERS[args.rest_provider](files, args.rest_endpoint)
@@ -142,6 +169,25 @@ class Serve(_BkServe):
         elif args.rest_provider is not None:
             raise ValueError("rest-provider %r not recognized." % args.rest_provider)
 
+        config.autoreload = args.autoreload
+
+        if config.autoreload:
+            for f in files:
+                watch(f)
+
+        if args.warm or args.autoreload:
+            argvs = {f: args.args for f in files}
+            applications = build_single_handler_applications(files, argvs)
+            if args.autoreload:
+                with record_modules():
+                    for app in applications.values():
+                        doc = app.create_document()
+                        _cleanup_doc(doc)
+            else:
+                for app in applications.values():
+                    doc = app.create_document()
+                    _cleanup_doc(doc)
+                    
         config.session_history = args.session_history
         if args.rest_session_info:
             pattern = REST_PROVIDERS['param'](files, 'rest')
