@@ -2,8 +2,6 @@
 Various utilities for loading JS dependencies and rendering plots
 inside the Jupyter notebook.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import json
 import uuid
 import sys
@@ -19,7 +17,6 @@ from bokeh.core.json_encoder import serialize_json
 from bokeh.core.templates import MACROS
 from bokeh.document import Document
 from bokeh.embed import server_document
-from bokeh.embed.bundle import bundle_for_objs_and_resources
 from bokeh.embed.elements import div_for_render_item, script_for_render_items
 from bokeh.embed.util import standalone_docs_json_and_render_items
 from bokeh.embed.wrappers import wrap_in_script_tag
@@ -39,7 +36,7 @@ except Exception:
 from ..compiler import require_components
 from .embed import embed_state
 from .model import add_to_doc, diff
-from .resources import _env
+from .resources import Bundle, Resources, _env, bundle_resources
 from .server import _server_url, _origin_url, get_server
 from .state import state
 
@@ -52,6 +49,10 @@ LOAD_MIME = 'application/vnd.holoviews_load.v0+json'
 EXEC_MIME = 'application/vnd.holoviews_exec.v0+json'
 HTML_MIME = 'text/html'
 
+def _jupyter_server_extension_paths():
+    return [{"module": "panel.io.jupyter_server_extension"}]
+
+
 def push(doc, comm, binary=True):
     """
     Pushes events stored on the document across the provided comm.
@@ -66,16 +67,44 @@ def push(doc, comm, binary=True):
         comm.send(json.dumps(header))
         comm.send(buffers=[payload])
 
+
+def push_on_root(ref):
+    if ref not in state._views:
+        return
+    (self, root, doc, comm) = state._views[ref]
+    if comm and 'embedded' not in root.tags:
+        push(doc, comm)
+
+
+def push_notebook(*objs):
+    """
+    A utility for pushing updates to the frontend given a Panel
+    object.  This is required when modifying any Bokeh object directly
+    in a notebook session.
+
+    Arguments
+    ---------
+    objs: panel.viewable.Viewable
+    """
+    for obj in objs:
+        for ref in obj._models:
+            push_on_root(ref)
+
+
 DOC_NB_JS = _env.get_template("doc_nb_js.js")
 AUTOLOAD_NB_JS = _env.get_template("autoload_panel_js.js")
 NB_TEMPLATE_BASE = _env.get_template('nb_template.html')
 
 def _autoload_js(bundle, configs, requirements, exports, skip_imports, ipywidget, load_timeout=5000):
+    config = {'packages': {}, 'paths': {}, 'shim': {}}
+    for conf in configs:
+        for key, c in conf.items():
+            config[key].update(c)
     return AUTOLOAD_NB_JS.render(
         bundle    = bundle,
         force     = True,
         timeout   = load_timeout,
-        configs   = configs,
+        config    = config,
         requirements = requirements,
         exports   = exports,
         skip_imports = skip_imports,
@@ -210,9 +239,10 @@ def load_notebook(inline=True, load_timeout=5000):
     resources = INLINE if inline else CDN
     prev_resources = settings.resources(default="server")
     user_resources = settings.resources._user_value is not _Unset
+    resources = Resources.from_bokeh(resources)
     try:
-        settings.resources = 'inline' if inline else 'cdn'
-        bundle = bundle_for_objs_and_resources(None, resources)
+        bundle = bundle_resources(resources)
+        bundle = Bundle.from_bokeh(bundle)
         configs, requirements, exports, skip_imports = require_components()
         ipywidget = 'ipywidgets_bokeh' in sys.modules
         bokeh_js = _autoload_js(bundle, configs, requirements, exports,
@@ -336,7 +366,7 @@ def ipywidget(obj, **kwargs):
     -------
     Returns an ipywidget model which renders the Panel object.
     """
-    from jupyter_bokeh import BokehModel
+    from jupyter_bokeh.widgets   import BokehModel
     from ..pane import panel
     model = panel(obj, **kwargs).get_root()
     widget = BokehModel(model, combine_events=True)
@@ -345,7 +375,10 @@ def ipywidget(obj, **kwargs):
         def view_count_changed(change, current=[model]):
             new_model = None
             if change['old'] > 0 and change['new'] == 0 and current:
-                obj._cleanup(current[0])
+                try:
+                    obj._cleanup(current[0])
+                except Exception:
+                    pass
                 current[:] = []
             elif (change['old'] == 0 and change['new'] > 0 and
                   (not current or current[0] is not model)):
