@@ -5,11 +5,182 @@ import param
 import weakref
 import sys
 
+import bokeh.core.properties as bp
+import param as pm
+import bokeh
+
+from bokeh.model import DataModel
+from bokeh.models import ColumnDataSource, CustomJS, Model as BkModel
+
 from .config import config
-from .reactive import Reactive
+from .io.notebook import push_on_root
+from .models import ReactiveHTML
+from .reactive import Reactive, Syncable
 from .viewable import Viewable
 
-from bokeh.models import (CustomJS, Model as BkModel)
+
+class Parameterized(bokeh.core.property.bases.Property):
+    """ Accept a Parameterized object.
+
+    This property only exists to support type validation, e.g. for "accepts"
+    clauses. It is not serializable itself, and is not useful to add to
+    Bokeh models directly.
+
+    """
+    def validate(self, value, detail=True):
+        super().validate(value, detail)
+
+        if isinstance(value, param.Parameterized):
+            return
+
+        msg = "" if not detail else f"expected param.Parameterized, got {value!r}"
+        raise ValueError(msg)
+
+
+
+_DATA_MODELS = weakref.WeakKeyDictionary()
+
+PARAM_MAPPING = {
+    pm.Array: lambda p, kwargs: bp.Array(bp.Any, **kwargs),
+    pm.Boolean: lambda p, kwargs: bp.Bool(**kwargs),
+    pm.CalendarDate: lambda p, kwargs: bp.Date(**kwargs),
+    pm.CalendarDateRange: lambda p, kwargs: bp.Tuple(bp.Date, bp.Date, **kwargs),
+    pm.ClassSelector: lambda p, kwargs: (
+        (bp.Instance(DataModel, **kwargs), [(Parameterized, create_linked_datamodel)])
+        if isinstance(p.class_, type) and issubclass(p.class_, param.Parameterized) else
+        bp.Any(**kwargs)
+    ),
+    pm.Color: lambda p, kwargs: bp.Color(**kwargs),
+    pm.DataFrame: lambda p, kwargs: (
+        bp.ColumnData(bp.Any, bp.Seq(bp.Any), **kwargs),
+        [(bp.PandasDataFrame, lambda x: ColumnDataSource._data_from_df(x))]
+    ),
+    pm.DateRange: lambda p, kwargs: bp.Tuple(bp.Datetime, bp.Datetime, **kwargs),
+    pm.Date: lambda p, kwargs: bp.Datetime(**kwargs),
+    pm.Dict: lambda p, kwargs: bp.Dict(bp.String, bp.Any, **kwargs),
+    pm.Event: lambda p, kwargs: bp.Bool(**kwargs),
+    pm.Integer: lambda p, kwargs: bp.Int(**kwargs),
+    pm.List: lambda p, kwargs: bp.List(bp.Any, **kwargs),
+    pm.Number: lambda p, kwargs: bp.Float(**kwargs),
+    pm.NumericTuple: lambda p, kwargs: bp.Tuple(*(bp.Float for p in p.length), **kwargs),
+    pm.Range: lambda p, kwargs: bp.Tuple(bp.Float, bp.Float, **kwargs),
+    pm.String: lambda p, kwargs: bp.String(**kwargs),
+    pm.Tuple: lambda p, kwargs: bp.Tuple(*(bp.Any for p in p.length), **kwargs),
+}
+
+
+def construct_data_model(parameterized, name=None, ignore=[], types={}):
+    """
+    Dynamically creates a Bokeh DataModel class from a Parameterized
+    object.
+
+    Arguments
+    ---------
+    parameterized: param.Parameterized
+        The Parameterized class or instance from which to create the
+        DataModel
+    name: str or None
+        Name of the dynamically created DataModel class
+    ignore: list(str)
+        List of parameters to ignore.
+    types: dict
+        A dictionary mapping from parameter name to a Parameter type,
+        making it possible to override the default parameter types.
+
+    Returns
+    -------
+    DataModel
+    """
+
+    properties = {}
+    for pname in parameterized.param:
+        if pname in ignore:
+            continue
+        p = parameterized.param[pname]
+        if p.precedence and p.precedence < 0:
+            continue
+        ptype = types.get(pname, type(p))
+        prop = PARAM_MAPPING.get(ptype)
+        if isinstance(parameterized, Syncable):
+            pname = parameterized._rename.get(pname, pname)
+        if pname == 'name' or pname is None:
+            continue
+        nullable = getattr(p, 'allow_None', False)
+        kwargs = {'default': p.default, 'help': p.doc}
+        if prop is None:
+            bk_prop, accepts = bp.Any(**kwargs), []
+        else:
+            bkp = prop(p, {} if nullable else kwargs)
+            bk_prop, accepts = bkp if isinstance(bkp, tuple) else (bkp, [])
+            if nullable:
+                bk_prop = bp.Nullable(bk_prop, **kwargs)
+        for bkp, convert in accepts:
+            bk_prop = bk_prop.accepts(bkp, convert)
+        properties[pname] = bk_prop
+    name = name or parameterized.name
+    return type(name, (DataModel,), properties)
+
+
+def create_linked_datamodel(obj, root=None):
+    """
+    Creates a Bokeh DataModel from a Parameterized class or instance
+    which automatically links the parameters bi-directionally.
+
+    Arguments
+    ---------
+    obj: param.Parameterized
+       The Parameterized class to create a linked DataModel for.
+
+    Returns
+    -------
+    DataModel instance linked to the Parameterized object.
+    """
+    if isinstance(obj, type) and issubclass(obj, param.Parameterized):
+        cls = obj
+    elif isinstance(obj, param.Parameterized):
+        cls = type(obj)
+    else:
+        raise TypeError('Can only create DataModel for Parameterized class or instance.')
+    if cls in _DATA_MODELS:
+        model = _DATA_MODELS[cls]
+    else:
+        _DATA_MODELS[cls] = model = construct_data_model(obj)
+    model = model(**dict(obj.param.get_param_values()))
+    _changing = []
+
+    def cb_bokeh(attr, old, new):
+        if attr in _changing:
+            return
+        try:
+            _changing.append(attr)
+            obj.param.set_param(**{attr: new})
+        finally:
+            _changing.remove(attr)
+
+    def cb_param(*events):
+        update = {
+            event.name: event.new for event in events
+            if event.name not in _changing
+        }
+        try:
+            _changing.extend(list(update))
+            model.update(**update)
+            tags = [tag for tag in model.tags if tag.startswith('__ref:')]
+            if root:
+                push_on_root(root.ref['id'])
+            elif tags:
+                ref = tags[0].split('__ref:')[-1]
+                push_on_root(ref)
+        finally:
+            for attr in update:
+                _changing.remove(attr)
+
+    for p in obj.param:
+        model.on_change(p, cb_bokeh)
+
+    obj.param.watch(cb_param, list(obj.param))
+
+    return model
 
 
 class Callback(param.Parameterized):
@@ -90,14 +261,16 @@ class Callback(param.Parameterized):
         if not linkable:
             return
 
-        found = [(link, src, getattr(link, 'target', None)) for src in linkable
-                 for link in cls.registry.get(src, [])
-                 if not link._requires_target or link.target in linkable]
+        found = [
+            (link, src, getattr(link, 'target', None)) for src in linkable
+            for link in cls.registry.get(src, [])
+            if not link._requires_target or link.target in linkable
+            or isinstance(link.target, param.Parameterized)
+        ]
 
         arg_overrides = {}
         if 'holoviews' in sys.modules:
             from .pane.holoviews import HoloViews, generate_panel_bokeh_map
-
             hv_views = root_view.select(HoloViews)
             map_hve_bk = generate_panel_bokeh_map(root_model, hv_views)
             for src in linkable:
@@ -117,15 +290,16 @@ class Callback(param.Parameterized):
 
         ref = root_model.ref['id']
         callbacks = []
-        for link, src, tgt in found:
+        for (link, src, tgt) in found:
             cb = cls._callbacks[type(link)]
             if ((src is None or ref not in getattr(src, '_models', [ref])) or
                 (getattr(link, '_requires_target', False) and tgt is None) or
                 (tgt is not None and ref not in getattr(tgt, '_models', [ref]))):
                 continue
             overrides = arg_overrides.get(id(link), {})
-            callbacks.append(cb(root_model, link, src, tgt,
-                                arg_overrides=overrides))
+            callbacks.append(
+                cb(root_model, link, src, tgt, arg_overrides=overrides)
+            )
         return callbacks
 
 
@@ -195,6 +369,8 @@ class Link(Callback):
 
 class CallbackGenerator(object):
 
+    error = False
+
     def __init__(self, root_model, link, source, target=None, arg_overrides={}):
         self.root_model = root_model
         self.link = link
@@ -204,11 +380,16 @@ class CallbackGenerator(object):
         self.validate()
         specs = self._get_specs(link, source, target)
         for src_spec, tgt_spec, code in specs:
+            if src_spec[1] and target is not None and src_spec[1].startswith('event:') and not tgt_spec[1]:
+                continue
             try:
                 self._init_callback(root_model, link, source, src_spec, target, tgt_spec, code)
             except Exception:
-                pass
-                
+                if self.error:
+                    raise
+                else:
+                    pass
+
     @classmethod
     def _resolve_model(cls, root_model, obj, model_spec):
         """
@@ -248,6 +429,8 @@ class CallbackGenerator(object):
             model, _ = obj._models[root_model.ref['id']]
         elif isinstance(obj, BkModel):
             model = obj
+        elif isinstance(obj, param.Parameterized):
+            model = create_linked_datamodel(obj, root_model)
         if model_spec is not None:
             for spec in model_spec.split('.'):
                 model = getattr(model, spec)
@@ -305,6 +488,15 @@ class CallbackGenerator(object):
                     if isinstance(v, BkModel) and k not in references:
                         references[k] = v
 
+        # Handle links with ReactiveHTML DataModel
+        if isinstance(src_model, ReactiveHTML):
+            if src_spec[1] in src_model.data.properties():
+                references['source'] = src_model = src_model.data
+
+        if isinstance(tgt_model, ReactiveHTML):
+            if tgt_spec[1] in tgt_model.data.properties():
+                references['target'] = tgt_model = tgt_model.data
+
         self._initialize_models(link, source, src_model, src_spec[1], target, tgt_model, tgt_spec[1])
         self._process_references(references)
 
@@ -327,7 +519,14 @@ class CallbackGenerator(object):
             reverse_references['target'] = src_model
             tgt_cb = CustomJS(args=reverse_references, code=code, tags=[link_id])
             changes, events = self._get_triggers(link, tgt_spec)
+            properties = tgt_model.properties()
             for ch in changes:
+                if ch not in properties:
+                    msg = f"Could not link non-existent property '{ch}' on {tgt_model} model"
+                    if self.error:
+                        raise ValueError(msg)
+                    else:
+                        self.param.warning(msg)
                 tgt_model.js_on_change(ch, tgt_cb)
             for ev in events:
                 tgt_model.js_on_event(ev, tgt_cb)
@@ -501,7 +700,8 @@ class JSLinkCallbackGenerator(JSCallbackGenerator):
             else:
                 value = getattr(src_model, src_spec)
             if value and hasattr(tgt_model, tgt_spec):
-                setattr(tgt_model, tgt_spec, value)
+                if tgt_spec != 'value_throttled':
+                    setattr(tgt_model, tgt_spec, value)
         if tgt_model is None and not link.code:
             raise ValueError('Model could not be resolved on target '
                              '%s and no custom code was specified.' %
@@ -539,7 +739,7 @@ class JSLinkCallbackGenerator(JSCallbackGenerator):
                 loading_spinner=config.loading_spinner
             )
         else:
-            if src_spec.startswith('event:'):
+            if src_spec and src_spec.startswith('event:'):
                 template = self._event_link_template
             else:
                 template = self._link_template
