@@ -10,15 +10,19 @@ from contextlib import contextmanager
 import param
 
 from bokeh.models import CustomJS, LayoutDOM, Model, Spacer as BkSpacer
+from bokeh.themes import Theme
 
 from ..io import remove_root
 from ..io.notebook import push
+from ..util import escape
 from ..viewable import Layoutable
 from .base import PaneBase
 from .ipywidget import IPyWidget
 from .markup import HTML
 from .image import PNG
 
+FOLIUM_BEFORE = '<div style="width:100%;"><div style="position:relative;width:100%;height:0;padding-bottom:60%;">'
+FOLIUM_AFTER = '<div style="width:100%;height:100%"><div style="position:relative;width:100%;height:100%;padding-bottom:0%;">'
 
 @contextmanager
 def _wrap_callback(cb, wrapped, doc, comm, callbacks):
@@ -28,7 +32,7 @@ def _wrap_callback(cb, wrapped, doc, comm, callbacks):
     replaces the wrapped callback with the real one while the callback
     is exectuted to ensure the callback can be removed as usual.
     """
-    hold = doc._hold
+    hold = doc.callbacks.hold_value
     doc.hold('combine')
     if wrapped in callbacks:
         index = callbacks.index(wrapped)
@@ -46,8 +50,13 @@ class Bokeh(PaneBase):
     Bokeh panes allow including any Bokeh model in a panel.
     """
 
+    theme = param.ClassSelector(default=None, class_=(Theme, str), doc="""
+        Bokeh theme to apply to the plot.""")
+
     priority = 0.8
 
+    _rename = {'theme': None}
+
     @classmethod
     def applies(cls, obj):
         return isinstance(obj, LayoutDOM)
@@ -109,6 +118,10 @@ class Bokeh(PaneBase):
             remove_root(model, doc)
 
         self._models[ref] = (model, parent)
+
+        if self.theme:
+            doc.theme = self.theme
+
         return model
 
 
@@ -208,7 +221,7 @@ class Matplotlib(PNG, IPyWidget):
         w, h = self.object.get_size_inches()
         return int(w*72), int(h*72)
 
-    def _img(self):
+    def _data(self):
         self.object.set_dpi(self.dpi)
         b = BytesIO()
 
@@ -300,3 +313,10 @@ class Folium(HTML):
     def applies(cls, obj):
         return (getattr(obj, '__module__', '').startswith('folium.') and
                 hasattr(obj, "_repr_html_"))
+
+    def _get_properties(self):
+        properties = super()._get_properties()
+        text = '' if self.object is None else self.object
+        if hasattr(text, '_repr_html_'):
+            text = text._repr_html_().replace(FOLIUM_BEFORE, FOLIUM_AFTER)
+        return dict(properties, text=escape(text))
