@@ -7,36 +7,27 @@ import json
 import base64
 import zipfile
 
-try:
-    from urllib.request import urlopen
-except ImportError: # python 2
-    from urllib import urlopen
-
+from abc import abstractmethod
 from six import string_types
+from urllib.request import urlopen
 
 import param
 import numpy as np
 
-from pyviz_comms import JupyterComm
-
-from .enums import PRESET_CMAPS
-from ..base import PaneBase, Pane
-from ...util import isfile
-
-if sys.version_info >= (2, 7):
-    base64encode = lambda x: base64.b64encode(x).decode('utf-8')
-else:
-    base64encode = lambda x: x.encode('base64')
-
 from bokeh.util.serialization import make_globally_unique_id
 from bokeh.models import LinearColorMapper
-from abc import abstractmethod
+from pyviz_comms import JupyterComm
+
+from ...util import isfile, lazy_load
+from ..base import PaneBase, Pane
+from .enums import PRESET_CMAPS
+
+base64encode = lambda x: base64.b64encode(x).decode('utf-8')
+
 
 class AbstractVTK(PaneBase):
 
-    __abstract = True
-
-    axes = param.Dict(doc="""
+    axes = param.Dict(default={}, doc="""
         Parameters of the axes to construct in the 3d view.
 
         Must contain at least ``xticker``, ``yticker`` and ``zticker``.
@@ -59,18 +50,21 @@ class AbstractVTK(PaneBase):
                 Defines the axes lines opacity.
     """)
 
-    camera = param.Dict(doc="""State of the rendered VTK camera.""")
+    camera = param.Dict(doc="""
+      State of the rendered VTK camera.""")
 
-    color_mappers = param.List(doc="""Color mapper of the actor in the scene""")
+    color_mappers = param.List(doc="""
+      Color mapper of the actor in the scene""")
 
     orientation_widget = param.Boolean(default=False, doc="""
-        Activate/Deactivate the orientation widget display.""")
+      Activate/Deactivate the orientation widget display.""")
 
-    interactive_orientation_widget = param.Boolean(default=True, constant=True, doc="""
-    """)
+    interactive_orientation_widget = param.Boolean(default=True, constant=True)
+
+    __abstract = True
 
     def _process_param_change(self, msg):
-        msg = super(AbstractVTK, self)._process_param_change(msg)
+        msg = super()._process_param_change(msg)
         if 'axes' in msg and msg['axes'] is not None:
             VTKAxes = getattr(sys.modules['panel.models.vtk'], 'VTKAxes')
             axes = msg['axes']
@@ -85,7 +79,7 @@ class AbstractVTK(PaneBase):
                 msg['axes'] = VTKAxes(**axes)
             elif isinstance(axes, VTKAxes):
                 msg['axes'] = VTKAxes(**axes.properties_with_values())
-        super(AbstractVTK, self)._update_model(events, msg, root, model, doc, comm)
+        super()._update_model(events, msg, root, model, doc, comm)
 
 
 class SyncHelpers:
@@ -187,7 +181,6 @@ class VTK:
 
 class BaseVTKRenderWindow(AbstractVTK):
 
-    __abstract = True
 
     enable_keybindings = param.Boolean(default=False, doc="""
         Activate/Deactivate keys binding.
@@ -220,13 +213,15 @@ class BaseVTKRenderWindow(AbstractVTK):
         by using some custom javascript functions.
     """)
 
+    _applies_kw = True
+
     _rename = {'serialize_on_instantiation': None, 'serialize_all_data_arrays': None}
 
-    _applies_kw = True
+    __abstract = True
 
     def __init__(self, object, **params):
         self._debug_serializer = params.pop('debug_serializer', False)
-        super(BaseVTKRenderWindow, self).__init__(object, **params)
+        super().__init__(object, **params)
         import panel.pane.vtk.synchronizable_serializer as rws
         rws.initializeSerializers()
 
@@ -296,18 +291,18 @@ class BaseVTKRenderWindow(AbstractVTK):
         else:
             return Pane(self._construct_colorbars)
 
-
     def export_scene(self, filename='vtk_scene', all_data_arrays=False):
         if '.' not in filename:
             filename += '.synch'
         import panel.pane.vtk.synchronizable_serializer as rws
         context = rws.SynchronizationContext(serialize_all_data_arrays=all_data_arrays, debug=self._debug_serializer)
-        scene, arrays = self._serialize_ren_win(self.object, context, binary=True, compression=False)
+        scene, arrays, annotations = self._serialize_ren_win(self.object, context, binary=True, compression=False)
 
         with zipfile.ZipFile(filename, mode='w') as zf:
             zf.writestr('index.json', json.dumps(scene))
             for name, data in arrays.items():
                 zf.writestr('data/%s' % name, data, zipfile.ZIP_DEFLATED)
+            zf.writestr('annotations.json', json.dumps(annotations))
         return filename
 
     def _update_color_mappers(self):
@@ -315,28 +310,10 @@ class BaseVTKRenderWindow(AbstractVTK):
         if self.color_mappers != color_mappers:
             self.color_mappers = color_mappers
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
-        """
-        Should return the bokeh model to be rendered.
-        """
-        if 'panel.models.vtk' not in sys.modules:
-            if isinstance(comm, JupyterComm):
-                self.param.warning('VTKSynchronizedPlot was not imported on instantiation '
-                                   'and may not render in a notebook. Restart '
-                                   'the notebook kernel and ensure you load '
-                                   'it as part of the extension using:'
-                                   '\n\npn.extension(\'vtk\')\n')
-            from ...models.vtk import VTKSynchronizedPlot
-        else:
-            VTKSynchronizedPlot = getattr(sys.modules['panel.models.vtk'], 'VTKSynchronizedPlot')
-
-        return VTKSynchronizedPlot
-
     def _serialize_ren_win(self, ren_win, context, binary=False, compression=True, exclude_arrays=None):
         import panel.pane.vtk.synchronizable_serializer as rws
         if exclude_arrays is None:
             exclude_arrays = []
-        store_offscreen_rendering = ren_win.GetOffScreenRendering()
         ren_win.OffScreenRenderingOn() # to not pop a vtk windows
         ren_win.Modified()
         ren_win.Render()
@@ -345,9 +322,8 @@ class BaseVTKRenderWindow(AbstractVTK):
         arrays = {name: context.getCachedDataArray(name, binary=binary, compression=compression)
                     for name in context.dataArrayCache.keys()
                     if name not in exclude_arrays}
-        ren_win.Finalize()
-        ren_win.SetOffScreenRendering(store_offscreen_rendering)
-        return scene, arrays
+        annotations = context.getAnnotations()
+        return scene, arrays, annotations
 
     @staticmethod
     def _rgb2hex(r, g, b):
@@ -372,7 +348,7 @@ class VTKRenderWindow(BaseVTKRenderWindow):
     @classmethod
     def applies(cls, obj, **kwargs):
         serialize_on_instantiation = kwargs.get('serialize_on_instantiation', False)
-        return (super(VTKRenderWindow, cls).applies(obj, **kwargs) and
+        return (super().applies(obj, **kwargs) and
                 serialize_on_instantiation)
 
     def __init__(self, object=None, **params):
@@ -382,9 +358,10 @@ class VTKRenderWindow(BaseVTKRenderWindow):
             self._update()
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        VTKSynchronizedPlot = super(VTKRenderWindow, self)._get_model(doc, root=None, parent=None, comm=None)
-
-        props = self._process_param_change(self._init_properties())
+        VTKSynchronizedPlot = lazy_load(
+            'panel.models.vtk', 'VTKSynchronizedPlot', isinstance(comm, JupyterComm)
+        )
+        props = self._process_param_change(self._init_params())
         if self.object is not None:
             props.update(scene=self._scene, arrays=self._arrays, color_mappers=self.color_mappers)
         model = VTKSynchronizedPlot(**props)
@@ -404,14 +381,12 @@ class VTKRenderWindow(BaseVTKRenderWindow):
             serialize_all_data_arrays=self.serialize_all_data_arrays,
             debug=self._debug_serializer
         )
-        self._scene, self._arrays = self._serialize_ren_win(
+        self._scene, self._arrays, self._annotations = self._serialize_ren_win(
             self.object,
             context,
         )
         if model is not None:
-            model.update(rebuild=True)
-            model.update(arrays=self._arrays)
-            model.update(scene=self._scene)
+            model.update(rebuild=True, arrays=self._arrays, scene=self._scene, annotations=self._annotations)
 
 
 class VTKRenderWindowSynchronized(BaseVTKRenderWindow, SyncHelpers):
@@ -434,30 +409,28 @@ class VTKRenderWindowSynchronized(BaseVTKRenderWindow, SyncHelpers):
     @classmethod
     def applies(cls, obj, **kwargs):
         serialize_on_instantiation = kwargs.get('serialize_on_instantiation', False)
-        return (super(VTKRenderWindowSynchronized, cls).applies(obj, **kwargs) and
-                not serialize_on_instantiation)
+        return super().applies(obj, **kwargs) and not serialize_on_instantiation
 
     def __init__(self, object=None, **params):
         if object is None:
             object = self.make_ren_win()
-        super(VTKRenderWindowSynchronized, self).__init__(object, **params)
+        super().__init__(object, **params)
         self._contexts = {}
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        """
-        Should return the bokeh model to be rendered.
-        """
-        VTKSynchronizedPlot = super(VTKRenderWindowSynchronized, self)._get_model(doc, root=None, parent=None, comm=None)
+        VTKSynchronizedPlot = lazy_load(
+            'panel.models.vtk', 'VTKSynchronizedPlot', isinstance(comm, JupyterComm)
+        )
         import panel.pane.vtk.synchronizable_serializer as rws
         context = rws.SynchronizationContext(
             id_root=make_globally_unique_id(),
             serialize_all_data_arrays=self.serialize_all_data_arrays,
             debug=self._debug_serializer
         )
-        scene, arrays = self._serialize_ren_win(self.object, context)
+        scene, arrays, annotations = self._serialize_ren_win(self.object, context)
         self._update_color_mappers()
-        props = self._process_param_change(self._init_properties())
-        props.update(scene=scene, arrays=arrays, color_mappers=self.color_mappers)
+        props = self._process_param_change(self._init_params())
+        props.update(scene=scene, arrays=arrays, annotations=annotations, color_mappers=self.color_mappers)
         model = VTKSynchronizedPlot(**props)
 
         if root is None:
@@ -473,18 +446,17 @@ class VTKRenderWindowSynchronized(BaseVTKRenderWindow, SyncHelpers):
     def _cleanup(self, root):
         ref = root.ref['id']
         self._contexts.pop(ref, None)
-        super(VTKRenderWindowSynchronized, self)._cleanup(root)
+        super()._cleanup(root)
 
     def _update(self, ref=None, model=None):
         context = self._contexts[model.id]
-        scene, arrays = self._serialize_ren_win(
+        scene, arrays, annotations = self._serialize_ren_win(
             self.object,
             context,
             exclude_arrays=model.arrays_processed
         )
         context.checkForArraysToRelease()
-        model.update(arrays=arrays)
-        model.update(scene=scene)
+        model.update(arrays=arrays, scene=scene, annotations=annotations)
 
     def synchronize(self):
         self.param.trigger('object')
@@ -619,7 +591,7 @@ class VTKVolume(AbstractVTK):
     _updates = True
 
     def __init__(self, object=None, **params):
-        super(VTKVolume, self).__init__(object, **params)
+        super().__init__(object, **params)
         self._sub_spacing = self.spacing
         self._update()
 
@@ -628,32 +600,21 @@ class VTKVolume(AbstractVTK):
         if ((isinstance(obj, np.ndarray) and obj.ndim == 3) or
             any([isinstance(obj, k) for k in cls._serializers.keys()])):
             return True
-        elif 'vtk' not in sys.modules:
+        elif 'vtk' not in sys.modules and 'vtkmodules' not in sys.modules:
             return False
         else:
             import vtk
             return isinstance(obj, vtk.vtkImageData)
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        """
-        Should return the bokeh model to be rendered.
-        """
-        if 'panel.models.vtk' not in sys.modules:
-            if isinstance(comm, JupyterComm):
-                self.param.warning('VTKVolumePlot was not imported on instantiation '
-                                   'and may not render in a notebook. Restart '
-                                   'the notebook kernel and ensure you load '
-                                   'it as part of the extension using:'
-                                   '\n\npn.extension(\'vtk\')\n')
-            from ...models.vtk import VTKVolumePlot
-        else:
-            VTKVolumePlot = getattr(sys.modules['panel.models.vtk'], 'VTKVolumePlot')
+        VTKVolumePlot = lazy_load(
+            'panel.models.vtk', 'VTKVolumePlot', isinstance(comm, JupyterComm)
+        )
+        props = self._process_param_change(self._init_params())
+        if self._volume_data is not None:
+            props['data'] = self._volume_data
 
-        props = self._process_param_change(self._init_properties())
-        volume_data = self._volume_data
-
-        model = VTKVolumePlot(data=volume_data,
-                              **props)
+        model = VTKVolumePlot(**props)
         if root is None:
             root = model
         self._link_props(model, ['colormap', 'orientation_widget', 'camera', 'mapper', 'controller_expanded'], doc, root, comm)
@@ -662,13 +623,7 @@ class VTKVolume(AbstractVTK):
 
     def _update_object(self, ref, doc, root, parent, comm):
         self._legend = None
-        super(VTKVolume, self)._update_object(ref, doc, root, parent, comm)
-
-    def _init_properties(self):
-        return {k: v for k, v in self.param.get_param_values()
-                if v is not None and k not in [
-                    'default_layout', 'object', 'max_data_size', 'spacing', 'origin'
-                ]}
+        super()._update_object(ref, doc, root, parent, comm)
 
     def _get_object_dimensions(self):
         if isinstance(self.object, np.ndarray):
@@ -677,7 +632,7 @@ class VTKVolume(AbstractVTK):
             return self.object.GetDimensions()
 
     def _process_param_change(self, msg):
-        msg = super(VTKVolume, self)._process_param_change(msg)
+        msg = super()._process_param_change(msg)
         if self.object is not None:
             slice_params = {'slice_i':0, 'slice_j':1, 'slice_k':2}
             for k, v in msg.items():
@@ -689,7 +644,7 @@ class VTKVolume(AbstractVTK):
         return msg
 
     def _process_property_change(self, msg):
-        msg = super(VTKVolume, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if self.object is not None:
             slice_params = {'slice_i':0, 'slice_j':1, 'slice_k':2}
             for k, v in msg.items():
@@ -797,7 +752,7 @@ class VTKJS(AbstractVTK):
 
 
     def __init__(self, object=None, **params):
-        super(VTKJS, self).__init__(object, **params)
+        super().__init__(object, **params)
         self._vtkjs = None
 
     @classmethod
@@ -809,21 +764,12 @@ class VTKJS(AbstractVTK):
         """
         Should return the bokeh model to be rendered.
         """
-        if 'panel.models.vtk' not in sys.modules:
-            if isinstance(comm, JupyterComm):
-                self.param.warning('VTKPlot was not imported on instantiation '
-                                   'and may not render in a notebook. Restart '
-                                   'the notebook kernel and ensure you load '
-                                   'it as part of the extension using:'
-                                   '\n\npn.extension(\'vtk\')\n')
-            from ...models.vtk import VTKJSPlot
-        else:
-            VTKJSPlot = getattr(sys.modules['panel.models.vtk'], 'VTKJSPlot')
-
+        VTKJSPlot = lazy_load('panel.models.vtk', 'VTKJSPlot', isinstance(comm, JupyterComm))
+        props = self._process_param_change(self._init_params())
         vtkjs = self._get_vtkjs()
-        data = base64encode(vtkjs) if vtkjs is not None else vtkjs
-        props = self._process_param_change(self._init_properties())
-        model = VTKJSPlot(data=data, **props)
+        if vtkjs is not None:
+            props['data'] = base64encode(vtkjs)
+        model = VTKJSPlot(**props)
         if root is None:
             root = model
         self._link_props(model, ['camera', 'enable_keybindings', 'orientation_widget'], doc, root, comm)
