@@ -115,6 +115,9 @@ class Param(PaneBase):
     height = param.Integer(default=None, bounds=(0, None), doc="""
         Height of widgetbox the parameter widgets are displayed in.""")
 
+    hide_constant = param.Boolean(default=False, doc="""
+        Whether to hide widgets of constant parameters.""")
+
     initializer = param.Callable(default=None, doc="""
         User-supplied function that will be called on initialization,
         usually to update the default Parameter values of the
@@ -213,7 +216,8 @@ class Param(PaneBase):
                              type(layout).__name__)
         self.param.watch(self._update_widgets, [
             'object', 'parameters', 'name', 'display_threshold', 'expand_button',
-            'expand', 'expand_layout', 'widgets', 'show_labels', 'show_name'])
+            'expand', 'expand_layout', 'widgets', 'show_labels', 'show_name',
+            'hide_constant'])
         self._update_widgets()
 
     def __repr__(self, depth=0):
@@ -384,6 +388,8 @@ class Param(PaneBase):
         else:
             label = p_obj.label
         kw = dict(disabled=p_obj.constant, name=label)
+        if self.hide_constant:
+            kw['visible'] = not p_obj.constant
 
         value = getattr(self.object, p_name)
         if value is not None:
@@ -455,12 +461,20 @@ class Param(PaneBase):
 
         def link(change, watchers=[watcher]):
             updates = {}
+            if p_name not in self._widgets:
+                return
             widget = self._widgets[p_name]
             if change.what == 'constant':
                 updates['disabled'] = change.new
+                if self.hide_constant:
+                    updates['visible'] = not change.new
             elif change.what == 'precedence':
-                if (change.new < self.display_threshold and
-                    widget in self._widget_box.objects):
+                if change.new is change.old:
+                    return
+                elif change.new is None:
+                    self._rerender()
+                elif (change.new < self.display_threshold and
+                      widget in self._widget_box.objects):
                     self._widget_box.pop(widget)
                 elif change.new >= self.display_threshold:
                     self._rerender()
@@ -499,6 +513,10 @@ class Param(PaneBase):
                 return
             elif kw_widget.get('throttled', False) and hasattr(widget, 'value_throttled'):
                 updates['value_throttled'] = change.new
+                updates['value'] = change.new
+            elif isinstance(widget, Row) and len(widget) == 2:
+                updates['value'] = change.new
+                widget = widget[0]
             else:
                 updates['value'] = change.new
 
@@ -815,8 +833,8 @@ class ParamFunction(ParamMethod):
     priority = 0.6
 
     def _link_object_params(self):
-        deps = self.object._dinfo
-        dep_params = list(deps['dependencies']) + list(deps.get('kw', {}).values())
+        deps = getattr(self.object, '_dinfo', {})
+        dep_params = list(deps.get('dependencies', [])) + list(deps.get('kw', {}).values())
         grouped = defaultdict(list)
         for dep in dep_params:
             grouped[id(dep.owner)].append(dep)
@@ -836,7 +854,11 @@ class ParamFunction(ParamMethod):
 
     @classmethod
     def applies(cls, obj):
-        return isinstance(obj, types.FunctionType) and hasattr(obj, '_dinfo')
+        if isinstance(obj, types.FunctionType):
+            if hasattr(obj, '_dinfo'):
+                return True
+            return None
+        return False
 
 
 class JSONInit(param.Parameterized):
