@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 import sys
 import traceback as tb
@@ -13,6 +11,7 @@ from .pane import HoloViews, Pane, Markdown
 from .widgets import Button, Select
 from .param import Param
 from .util import param_reprs
+from .viewable import Viewer
 
 
 class PipelineError(RuntimeError):
@@ -53,7 +52,7 @@ def find_route(graph, current, target):
 
 def get_root(graph):
     """
-    Search for the root not by finding nodes without inputs.
+    Search for the root node by finding nodes without inputs.
     """
     # Find root node
     roots = []
@@ -104,10 +103,10 @@ def get_breadths(node, graph, depth=0, breadths=None):
 
 
 
-class Pipeline(param.Parameterized):
+class Pipeline(Viewer):
     """
     A Pipeline represents a directed graph of stages, which each
-    returns a panel object to render. A pipeline therefore represents
+    return a panel object to render. A pipeline therefore represents
     a UI workflow of multiple linear or branching stages.
 
     The Pipeline layout consists of a number of sub-components:
@@ -134,7 +133,7 @@ class Pipeline(param.Parameterized):
 
     The stages are declared using the add_stage method and must each
     be given a unique name. By default any stages will simply be
-    connected linearly, however an explicit graph can be declared using
+    connected linearly, but an explicit graph can be declared using
     the define_graph method.
     """
 
@@ -170,7 +169,7 @@ class Pipeline(param.Parameterized):
         except Exception:
             raise ImportError('Pipeline requires holoviews to be installed')
 
-        super(Pipeline, self).__init__(**params)
+        super().__init__(**params)
 
         # Initialize internal state
         self._stage = None
@@ -224,6 +223,9 @@ class Pipeline(param.Parameterized):
             self.add_stage(name, stage, **kwargs)
         self.define_graph(graph)
 
+    def __panel__(self):
+        return self.layout
+
     def _validate(self, stage):
         if any(stage is s for n, (s, kw) in self._stages.items()):
             raise ValueError('Stage %s is already in pipeline' % stage)
@@ -529,9 +531,6 @@ class Pipeline(param.Parameterized):
         )
         return plot
 
-    def _repr_mimebundle_(self, include=None, exclude=None):
-        return self.layout._repr_mimebundle_(include, exclude)
-
     #----------------------------------------------------------------
     # Public API
     #----------------------------------------------------------------
@@ -614,6 +613,7 @@ class Pipeline(param.Parameterized):
             raise ValueError('Graph is not fully traversable from stage: %s.'
                              % root)
 
+        reinit = root is not self._stage
         self._stage = root
         self._graph = graph
         self._route = [root]
@@ -622,6 +622,7 @@ class Pipeline(param.Parameterized):
                 Column(self.prev_selector, self.prev_button),
                 Column(self.next_selector, self.next_button)
             ]
-        self.stage[:] = [self._init_stage()]
+        if reinit:
+            self.stage[:] = [self._init_stage()]
         self._update_progress()
         self._update_button()
