@@ -28,12 +28,12 @@ from ..layout import Column, ListLike, GridSpec
 from ..models.comm_manager import CommManager
 from ..pane import panel as _panel, HTML, Str, HoloViews
 from ..pane.image import ImageBase
+from ..reactive import ReactiveHTML
 from ..util import url_path
-
 from ..viewable import ServableMixin, Viewable
 from ..widgets import Button
 from ..widgets.indicators import BooleanIndicator, LoadingSpinner
-from .theme import DefaultTheme, Theme
+from .theme import THEMES, DefaultTheme, Theme
 
 _server_info = (
     '<b>Running server:</b> <a target="_blank" href="https://localhost:{port}">'
@@ -69,6 +69,7 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         self.nb_template = nb_template or template
         self._render_items = OrderedDict()
         self._render_variables = {}
+        self._documents = []
         self._server = None
         self._layout = self._build_layout()
 
@@ -132,12 +133,20 @@ class BaseTemplate(param.Parameterized, ServableMixin):
     def _apply_root(self, name, viewable, tags):
         pass
 
+    def _server_destroy(self, session_context):
+        doc = session_context._document
+        self._documents.remove(doc)
+        if doc in state._locations:
+            del state._locations[doc]
+
     def _init_doc(self, doc=None, comm=None, title=None, notebook=False, location=True):
         doc = doc or _curdoc()
+        self._documents.append(doc)
         title = title or 'Panel Application'
         if location and self.location:
             loc = self._add_location(doc, location)
             doc.on_session_destroyed(loc._server_destroy)
+        doc.on_session_destroyed(self._server_destroy)
         doc.title = title
 
         # Initialize fake root. This is needed to ensure preprocessors
@@ -147,7 +156,7 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         preprocess_root = col.get_root(doc, comm)
         col._hooks.append(self._apply_hooks)
         ref = preprocess_root.ref['id']
-        objs = []
+        objs, models = [], []
 
         for name, (obj, tags) in self._render_items.items():
             if self._apply_hooks not in obj._hooks:
@@ -169,12 +178,14 @@ class BaseTemplate(param.Parameterized, ServableMixin):
             self._apply_root(name, model, tags)
             add_to_doc(model, doc, hold=bool(comm))
             objs.append(obj)
+            models.append(model)
 
         # Here we ensure that the preprocessor is run across all roots
         # and set up session cleanup hooks for the fake root.
-        state._fake_roots.append(ref)
+        state._fake_roots.append(ref) # Ensure no update is run
         state._views[ref] = (col, preprocess_root, doc, comm)
         col.objects = objs
+        preprocess_root.children[:] = models
         col._preprocess(preprocess_root)
         col._documents[doc] = preprocess_root
         doc.on_session_destroyed(col._server_destroy)
@@ -313,6 +324,24 @@ class BaseTemplate(param.Parameterized, ServableMixin):
 
 
 
+class TemplateActions(ReactiveHTML):
+    """
+    A component added to templates that allows triggering events such
+    as opening and closing a modal.
+    """
+
+    open_modal = param.Integer(default=0)
+
+    close_modal = param.Integer(default=0)
+
+    _template = "<div></div>"
+
+    _scripts = {
+        'open_modal': ["document.getElementById('pn-Modal').style.display = 'block'"],
+        'close_modal': ["document.getElementById('pn-Modal').style.display = 'none'"],
+    }
+
+
 class BasicTemplate(BaseTemplate):
     """
     BasicTemplate provides a baseclass for templates with a basic
@@ -344,14 +373,17 @@ class BasicTemplate(BaseTemplate):
     sidebar = param.ClassSelector(class_=ListLike, constant=True, doc="""
         A list-like container which populates the sidebar.""")
 
+    sidebar_width = param.Integer(330, doc="""
+        The width of the sidebar in pixels. Default is 330.""")
+
     modal = param.ClassSelector(class_=ListLike, constant=True, doc="""
         A list-like container which populates the modal""")
 
-    logo = param.String(constant=True, doc="""
+    logo = param.String(doc="""
         URI of logo to add to the header (if local file, logo is
-        base64 encoded as URI).""")
+        base64 encoded as URI). Default is '', i.e. not shown.""")
 
-    favicon = param.String(default=FAVICON_URL, constant=True, doc="""
+    favicon = param.String(default=FAVICON_URL, doc="""
         URI of favicon to add to the document head (if local file, favicon is
         base64 encoded as URI).""")
 
@@ -360,8 +392,11 @@ class BasicTemplate(BaseTemplate):
         meta settings and as the browser tab title.""")
 
     site = param.String(default="", doc="""
-        The name of the site. Will be shown in the header and link to the
-        root of the site. Default is '', i.e. not shown.""")
+        Name of the site. Will be shown in the header and link to the
+        'site_url'. Default is '', i.e. not shown.""")
+
+    site_url = param.String(default="/", doc="""
+        Url of the site and logo. Default is '/'.""")
 
     meta_description = param.String(doc="""
         A meta description to add to the document head for search
@@ -402,6 +437,8 @@ class BasicTemplate(BaseTemplate):
 
     location = param.Boolean(default=True, readonly=True)
 
+    _actions = param.ClassSelector(default=TemplateActions(), class_=TemplateActions)
+
     #############
     # Resources #
     #############
@@ -444,11 +481,14 @@ class BasicTemplate(BaseTemplate):
             params['modal'] = ListLike()
         else:
             params['modal'] = self._get_params(params['modal'], self.param.modal.class_)
+        if 'theme' in params and isinstance(params['theme'], str):
+            params['theme'] = THEMES[params['theme']]
         super().__init__(template=template, **params)
-        if self.busy_indicator:
-            state.sync_busy(self.busy_indicator)
         self._js_area = HTML(margin=0, width=0, height=0)
-        self._render_items['js_area'] = (self._js_area, [])
+        if '{{ embed(roots.js_area) }}' in template:
+            self._render_items['js_area'] = (self._js_area, [])
+        if '{{ embed(roots.actions) }}' in template:
+            self._render_items['actions'] = (self._actions, [])
         self._update_busy()
         self.main.param.watch(self._update_render_items, ['objects'])
         self.modal.param.watch(self._update_render_items, ['objects'])
@@ -461,6 +501,8 @@ class BasicTemplate(BaseTemplate):
 
     def _init_doc(self, doc=None, comm=None, title=None, notebook=False, location=True):
         title = title or self.title
+        if self.busy_indicator:
+            state.sync_busy(self.busy_indicator)
         self._update_vars()
         doc = super()._init_doc(doc, comm, title, notebook, location)
         if self.theme:
@@ -469,6 +511,12 @@ class BasicTemplate(BaseTemplate):
                 doc.theme = theme.bokeh_theme
         return doc
 
+    def _apply_hooks(self, viewable, root):
+        super()._apply_hooks(viewable, root)
+        theme = self._get_theme()
+        if theme and theme.bokeh_theme and root.document:
+            root.document.theme = theme.bokeh_theme
+
     def _get_theme(self):
         return self.theme.find_theme(type(self))()
 
@@ -580,6 +628,7 @@ class BasicTemplate(BaseTemplate):
         self._render_variables['app_title'] = self.title
         self._render_variables['meta_name'] = self.title
         self._render_variables['site_title'] = self.site
+        self._render_variables['site_url'] = self.site_url
         self._render_variables['meta_description'] = self.meta_description
         self._render_variables['meta_keywords'] = self.meta_keywords
         self._render_variables['meta_author'] = self.meta_author
@@ -611,6 +660,7 @@ class BasicTemplate(BaseTemplate):
         self._render_variables['header_background'] = self.header_background
         self._render_variables['header_color'] = self.header_color
         self._render_variables['main_max_width'] = self.main_max_width
+        self._render_variables['sidebar_width'] = self.sidebar_width
 
     def _update_busy(self):
         if self.busy_indicator:
@@ -657,29 +707,22 @@ class BasicTemplate(BaseTemplate):
         self._render_variables['header'] = any('header' in ts for ts in tags)
         self._render_variables['root_labels'] = labels
 
+    def _server_destroy(self, session_context):
+        super()._server_destroy(session_context)
+        if not self._documents and self.busy_indicator in state._indicators:
+            state._indicators.remove(self.busy_indicator)
+
     def open_modal(self):
         """
         Opens the modal area
         """
-        self._js_area.object = """
-        <script>
-          var modal = document.getElementById("pn-Modal");
-          modal.style.display = "block";
-        </script>
-        """
-        self._js_area.object = ""
+        self._actions.open_modal += 1
 
     def close_modal(self):
         """
         Closes the modal area
         """
-        self._js_area.object = """
-        <script>
-          var modal = document.getElementById("pn-Modal");
-          modal.style.display = "none";
-        </script>
-        """
-        self._js_area.object = ""
+        self._actions.close_modal += 1
 
     @staticmethod
     def _get_favicon_type(favicon):
