@@ -14,6 +14,8 @@ class BootstrapTemplate(BasicTemplate):
     """
     BootstrapTemplate
     """
+    sidebar_width = param.Integer(350, doc="""
+        The width of the sidebar in pixels. Default is 350.""")
 
     _css = pathlib.Path(__file__).parent / 'bootstrap.css'
 
