@@ -1,8 +1,6 @@
 """
 A module containing testing utilities and fixtures.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 import re
 import shutil
@@ -40,7 +38,7 @@ def dataframe():
     }, index=[1, 2, 3], columns=['int', 'float', 'str'])
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def hv_bokeh():
     import holoviews as hv
     hv.renderer('bokeh')
@@ -50,7 +48,7 @@ def hv_bokeh():
     hv.Store.current_backend = prev_backend
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def get_display_handle():
     cleanup = []
     def display_handle(model):
@@ -64,7 +62,7 @@ def get_display_handle():
             del state._handles[ref]
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def hv_mpl():
     import holoviews as hv
     hv.renderer('matplotlib')
@@ -74,7 +72,7 @@ def hv_mpl():
     hv.Store.current_backend = prev_backend
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def tmpdir(request, tmpdir_factory):
     name = request.node.name
     name = re.sub(r"[\W]", "_", name)
@@ -148,6 +146,14 @@ def multiple_apps_server_sessions():
             continue  # tests may already close this
 
 
+@pytest.fixture
+def with_curdoc():
+    old_curdoc = state.curdoc
+    state.curdoc = Document()
+    yield
+    state.curdoc = old_curdoc
+        
+
 @contextmanager
 def set_env_var(env_var, value):
     old_value = os.environ.get(env_var)
@@ -158,3 +164,13 @@ def set_env_var(env_var, value):
     else:
         os.environ[env_var] = old_value
 
+
+@pytest.fixture(autouse=True)
+def server_cleanup():
+    """
+    Clean up after test fails
+    """
+    yield
+    state.kill_all_servers()
+    state._indicators.clear()
+    state._locations.clear()
