@@ -1,26 +1,26 @@
 """
 Various general utilities used in the panel codebase.
 """
-from __future__ import absolute_import, division, unicode_literals
-
+import base64
 import datetime as dt
 import inspect
+import json
 import numbers
 import os
 import re
 import sys
+import urllib.parse as urlparse
 
+from collections.abc import MutableSequence, MutableMapping
 from collections import defaultdict, OrderedDict
+from contextlib import contextmanager
 from datetime import datetime
+from distutils.version import LooseVersion
+from html import escape # noqa
+from importlib import import_module
 from six import string_types
 
-try:  # python >= 3.3
-    from collections.abc import MutableSequence, MutableMapping
-except ImportError:
-    from collections import MutableSequence, MutableMapping
-
-from html import escape # noqa
-
+import bokeh
 import param
 import numpy as np
 
@@ -29,6 +29,8 @@ datetime_types = (np.datetime64, dt.datetime, dt.date)
 if sys.version_info.major > 2:
     unicode = str
 
+bokeh_version = LooseVersion(bokeh.__version__)
+
 
 def isfile(path):
     """Safe version of os.path.isfile robust to path length issues on Windows"""
@@ -41,11 +43,11 @@ def isfile(path):
 def isurl(obj, formats):
     if not isinstance(obj, string_types):
         return False
-    lower_string = obj.lower()
+    lower_string = obj.lower().split('?')[0].split('#')[0]
     return (
         lower_string.startswith('http://')
         or lower_string.startswith('https://')
-    ) and any(lower_string.endswith('.'+fmt) for fmt in formats)
+    ) and (formats is None or any(lower_string.endswith('.'+fmt) for fmt in formats))
 
 
 def is_dataframe(obj):
@@ -55,6 +57,13 @@ def is_dataframe(obj):
     return isinstance(obj, pd.DataFrame)
 
 
+def is_series(obj):
+    if 'pandas' not in sys.modules:
+        return False
+    import pandas as pd
+    return isinstance(obj, pd.Series)
+
+
 def hashable(x):
     if isinstance(x, MutableSequence):
         return tuple(x)
@@ -180,10 +189,16 @@ def param_reprs(parameterized, skip=None):
         default = parameterized.param[p].default
         equal = v is default
         if not equal:
-            try:
-                equal = bool(v==default)
-            except Exception:
-                equal = False
+            if isinstance(v, np.ndarray):
+                if isinstance(default, np.ndarray):
+                    equal = np.array_equal(v, default, equal_nan=True)
+                else:
+                    equal = False
+            else:
+                try:
+                    equal = bool(v==default)
+                except Exception:
+                    equal = False
 
         if equal: continue
         elif v is None: continue
@@ -230,7 +245,9 @@ def isdatetime(value):
     """
     Whether the array or scalar is recognized datetime type.
     """
-    if isinstance(value, np.ndarray):
+    if is_series(value) and len(value):
+        return isinstance(value.iloc[0], datetime_types)
+    elif isinstance(value, np.ndarray):
         return (value.dtype.kind == "M" or
                 (value.dtype.kind == "O" and len(value) and
                  isinstance(value[0], datetime_types)))
@@ -254,3 +271,116 @@ def value_as_date(value):
     elif isinstance(value, datetime):
         value = value.date()
     return value
+
+
+def is_number(s):
+    try:
+        float(s)
+        return True
+    except ValueError:
+        return False
+
+
+def parse_query(query):
+    """
+    Parses a url query string, e.g. ?a=1&b=2.1&c=string, converting
+    numeric strings to int or float types.
+    """
+    query = dict(urlparse.parse_qsl(query[1:]))
+    for k, v in list(query.items()):
+        if v.isdigit():
+            query[k] = int(v)
+        elif is_number(v):
+            query[k] = float(v)
+        elif v.startswith('[') or v.startswith('{'):
+            query[k] = json.loads(v)
+    return query
+
+
+def base64url_encode(input):
+    if isinstance(input, str):
+        input = input.encode("utf-8")
+    encoded = base64.urlsafe_b64encode(input).decode('ascii')
+    # remove padding '=' chars that cause trouble
+    return str(encoded.rstrip('='))
+
+
+def base64url_decode(input):
+    if isinstance(input, str):
+        input = input.encode("ascii")
+
+    rem = len(input) % 4
+
+    if rem > 0:
+        input += b"=" * (4 - rem)
+
+    return base64.urlsafe_b64decode(input)
+
+
+class classproperty(object):
+
+    def __init__(self, f):
+        self.f = f
+
+    def __get__(self, obj, owner):
+        return self.f(owner)
+
+
+def url_path(url):
+    return os.path.join(*os.path.join(*url.split('//')[1:]).split('/')[1:])
+
+
+# This functionality should be contributed to param
+# See https://github.com/holoviz/param/issues/379
+@contextmanager
+def edit_readonly(parameterized):
+    """
+    Temporarily set parameters on Parameterized object to readonly=False
+    to allow editing them.
+    """
+    params = parameterized.param.objects("existing").values()
+    readonlys = [p.readonly for p in params]
+    constants = [p.constant for p in params]
+    for p in params:
+        p.readonly = False
+        p.constant = False
+    try:
+        yield
+    except Exception:
+        raise
+    finally:
+        for (p, readonly) in zip(params, readonlys):
+            p.readonly = readonly
+        for (p, constant) in zip(params, constants):
+            p.constant = constant
+
+
+def lazy_load(module, model, notebook=False):
+    if module in sys.modules:
+        return getattr(sys.modules[module], model)
+    if notebook:
+        ext = module.split('.')[-1]
+        param.main.param.warning(f'{model} was not imported on instantiation '
+                                 'and may not render in a notebook. Restart '
+                                 'the notebook kernel and ensure you load '
+                                 'it as part of the extension using:'
+                                 f'\n\npn.extension(\'{ext}\')\n')
+    return getattr(import_module(module), model)
+
+
+def updating(fn):
+    def wrapped(self, *args, **kwargs):
+        updating = self._updating
+        self._updating = True
+        try:
+            fn(self, *args, **kwargs)
+        finally:
+            self._updating = updating
+    return wrapped
+
+
+def clone_model(bokeh_model, include_defaults=False, include_undefined=False):
+    properties = bokeh_model.properties_with_values(
+        include_defaults=include_defaults, include_undefined=include_undefined
+    )
+    return type(bokeh_model)(**properties)
