@@ -2,8 +2,6 @@
 Defines Button and button-like widgets which allow triggering events
 or merely toggling between on-off states.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 from functools import partial
 
 import param
@@ -12,13 +10,16 @@ from bokeh.models import (
     Button as _BkButton, Toggle as _BkToggle, Dropdown as _BkDropdown
 )
 
+from bokeh.events import (MenuItemClick, ButtonClick)
+
 from .base import Widget
 
 
+BUTTON_TYPES = ['default', 'primary', 'success', 'warning', 'danger']
+
 class _ButtonBase(Widget):
 
-    button_type = param.ObjectSelector(default='default', objects=[
-        'default', 'primary', 'success', 'warning', 'danger'])
+    button_type = param.ObjectSelector(default='default', objects=BUTTON_TYPES)
 
     _rename = {'name': 'label'}
 
@@ -32,7 +33,7 @@ class _ClickButton(_ButtonBase):
     _event = 'button_click'
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        model = super(_ClickButton, self)._get_model(doc, root, parent, comm)
+        model = super()._get_model(doc, root, parent, comm)
         ref = (root or model).ref['id']
         model.on_click(partial(self._server_click, doc, ref))
         return model
@@ -89,21 +90,36 @@ class Button(_ClickButton):
 
     clicks = param.Integer(default=0)
 
-    _rename = {'clicks': None, 'name': 'label'}
+    value = param.Event()
+
+    _rename = {'clicks': None, 'name': 'label', 'value': None}
+
+    _target_transforms = {'event:button_click': None, 'value': None}
 
     _widget_type = _BkButton
 
+    @property
+    def _linkable_params(self):
+        return super()._linkable_params + ['value']
+
+    def jslink(self, target, code=None, args=None, bidirectional=False, **links):
+        links = {'event:'+self._event if p == 'value' else p: v for p, v in links.items()}
+        super().jslink(target, code, args, bidirectional, **links)
+
+    jslink.__doc__ = Widget.jslink.__doc__
+
     def _server_click(self, doc, ref, event):
+        processing = bool(self._events)
         self._events.update({"clicks": self.clicks+1})
-        if not self._processing:
-            self._processing = True
+        self.param.trigger('value')
+        if not processing:
             if doc.session_context:
-                doc.add_timeout_callback(partial(self._change_event, doc), self._debounce)
+                doc.add_timeout_callback(partial(self._change_coroutine, doc), self._debounce)
             else:
                 self._change_event(doc)
 
     def _process_property_change(self, msg):
-        msg = super(Button, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'clicks' in msg:
             msg['clicks'] = self.clicks + 1
         return msg
@@ -150,10 +166,13 @@ class MenuButton(_ClickButton):
         self.param.watch(callback, 'clicked', onlychanged=False)
 
     def _server_click(self, doc, ref, event):
-        self._events.update({"clicked": event.item})
-        if not self._processing:
-            self._processing = True
+        processing = bool(self._events)
+        if isinstance(event, MenuItemClick):
+            self._events.update({"clicked": event.item})
+        elif isinstance(event, ButtonClick):
+            self._events.update({"clicked": self.name})
+        if not processing:
             if doc.session_context:
-                doc.add_timeout_callback(partial(self._change_event, doc), self._debounce)
+                doc.add_timeout_callback(partial(self._change_coroutine, doc), self._debounce)
             else:
                 self._change_event(doc)
