@@ -1,3 +1,4 @@
+import os
 import sys
 
 from math import pi
@@ -7,19 +8,22 @@ import param
 
 from bokeh.plotting import figure
 from bokeh.models import ColumnDataSource
+from tqdm.asyncio import tqdm as _tqdm
 
+from ..layout import Column, Row
 from ..models import (
     HTML, Progress as _BkProgress, TrendIndicator as _BkTrendIndicator
 )
+from ..pane.markup import Str
 from ..reactive import SyncableData
 from ..util import escape, updating
+from ..viewable import Viewable
 from .base import Widget
 
 RED   = "#d9534f"
 GREEN = "#5cb85c"
 BLUE  = "#428bca"
 
-
 class Indicator(Widget):
     """
     Indicator is a baseclass for widgets which indicate some state.
@@ -55,10 +59,12 @@ class BooleanStatus(BooleanIndicator):
     value = param.Boolean(default=False, doc="""
         Whether the indicator is active or not.""")
 
-    _widget_type = HTML
-
     _rename = {'color': None}
 
+    _source_transforms = {'value': None}
+
+    _widget_type = HTML
+
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
         value = msg.pop('value', None)
@@ -85,10 +91,12 @@ class LoadingSpinner(BooleanIndicator):
     value = param.Boolean(default=False, doc="""
         Whether the indicator is active or not.""")
 
-    _widget_type = HTML
-
     _rename = {'color': None, 'bgcolor': None}
 
+    _source_transforms = {'value': None}
+
+    _widget_type = HTML
+
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
         value = msg.pop('value', None)
@@ -161,6 +169,12 @@ class Number(ValueIndicator):
 
     _rename = {}
 
+    _source_transforms = {
+        'value': None, 'colors': None, 'default_color': None,
+        'font_size': None, 'format': None, 'nan_format': None,
+        'title_size': None
+    }
+
     _widget_type = HTML
 
     def _process_param_change(self, msg):
@@ -202,6 +216,10 @@ class String(ValueIndicator):
 
     _rename = {}
 
+    _source_transforms = {
+        'value': None, 'default_color': None, 'font_size': None, 'title_size': None
+    }
+
     _widget_type = HTML
 
     def _process_param_change(self, msg):
@@ -272,6 +290,14 @@ class Gauge(ValueIndicator):
 
     _rename = {}
 
+    _source_transforms = {
+        'annulus_width': None, 'bounds': None, 'colors': None,
+        'custom_opts': None, 'end_angle': None, 'format': None,
+        'num_splits': None, 'show_ticks': None, 'show_labels': None,
+        'start_angle': None, 'tooltip_format': None, 'title_size': None,
+        'value': None
+    }
+
     @property
     def _widget_type(self):
         if 'panel.models.echarts' not in sys.modules:
@@ -636,6 +662,11 @@ class Trend(SyncableData, Indicator):
         super()._update_cds(*events)
         self._trigger_auto_values()
 
+    def _update_data(self, data):
+        if isinstance(data, _BkTrendIndicator):
+            return
+        super()._update_data(data)
+
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
         ys = self._data.get(self.plot_y, [])
@@ -651,3 +682,165 @@ class Trend(SyncableData, Indicator):
             else:
                 msg['value_change'] = 0
         return msg
+
+
+MARGIN = {
+    "text_pane": {"column": (5, 10, 0, 10), "row": (0, 10, 0, 10)},
+    "progress": {"column": (0, 10, 5, 10), "row": (12, 10, 0, 10)},
+}
+
+
+
+class ptqdm(_tqdm):
+
+    def __init__(self, *args, **kwargs):
+        self._indicator = kwargs.pop('indicator')
+        super().__init__(*args, **kwargs)
+
+    def display(self, msg=None, pos=None, bar_style=None):
+        super().display(msg, pos)
+        style = self._indicator.text_pane.style or {}
+        color = self.colour or 'black'
+        self._indicator.text_pane.style = dict(style, color=color)
+        if self.total is not None and self.n is not None:
+            self._indicator.max = int(self.total) # Can be numpy.int64
+            self._indicator.value = int(self.n)
+            self._indicator.text = self._to_text(**self.format_dict)
+        return True
+
+    def _to_text(self, n, total, **kwargs):
+        return self.format_meter(n, total, **{**kwargs, "ncols": 0})
+
+    def close(self):
+        super().close()
+        if not self.leave:
+            self._indicator.reset()
+        return _tqdm
+
+
+class Tqdm(Indicator):
+
+    layout = param.ClassSelector(class_=(Column, Row), precedence=-1, constant=True, doc="""
+        The layout for the text and progress indicator.""",)
+
+    max = param.Integer(default=100, doc="""
+        The maximum value of the progress indicator.""")
+
+    progress = param.ClassSelector(class_=Progress, precedence=-1, doc="""
+        The Progress indicator used to display the progress.""",)
+
+    text = param.String(default='', doc="""
+        The current tqdm style progress text.""")
+
+    text_pane = param.ClassSelector(class_=Str, precedence=-1, doc="""
+        The pane to display the text to.""")
+
+    value = param.Integer(default=0, bounds=(0, None), doc="""
+        The current value of the progress bar. If set to None the progress
+        bar will be indeterminate and animate depending on the active
+        parameter.""")
+
+    margin = param.Parameter(default=0, doc="""
+        Allows to create additional space around the component. May
+        be specified as a two-tuple of the form (vertical, horizontal)
+        or a four-tuple (top, right, bottom, left).""")
+
+    width = param.Integer(default=400, bounds=(0, None), doc="""
+        The width of the component (in pixels). This can be either
+        fixed or preferred width, depending on width sizing policy.""")
+
+    write_to_console = param.Boolean(default=False, doc="""
+        Whether or not to also write to the console.""")
+
+    _layouts = {Row: 'row', Column: 'column'}
+
+    _rename = {'value': None, 'min': None, 'max': None, 'text': None}
+
+    def __init__(self, **params):
+        layout = params.pop('layout', 'column')
+        layout = self._layouts.get(layout, layout) 
+        if "text_pane" not in params:
+            sizing_mode = 'stretch_width' if layout == 'column' else 'fixed'
+            params["text_pane"] = Str(
+                None, min_height=20, min_width=280, sizing_mode=sizing_mode,
+                margin=MARGIN["text_pane"][layout],
+            )
+        if "progress" not in params:
+            params["progress"] = Progress(
+                active=False,
+                sizing_mode="stretch_width",
+                min_width=100,
+                margin=MARGIN["progress"][layout],
+            )
+
+        layout_params = {p: params.get(p, getattr(self, p))  for p in Viewable.param}
+        if layout == 'row' or layout is Row:
+            params['layout'] = Row(
+                params['progress'], params['text_pane'], **layout_params
+            )
+        else:
+            params['layout'] = Column(
+                params['text_pane'], params['progress'], **layout_params
+            )
+        super().__init__(**params)
+
+        self.param.watch(self._update_layout, list(Viewable.param))
+
+        if self.value == 0:
+            # Hack: to give progress the initial look
+            self.progress.max = 100000
+            self.progress.value = 1
+        else:
+            self.progress.max = self.max
+            self.progress.value = self.value
+        self.text_pane.object = self.text
+
+    def _get_model(self, doc, root=None, parent=None, comm=None):
+        model = self.layout._get_model(doc, root, parent, comm)
+        if root is None:
+            root = model
+        self._models[root.ref['id']] = (model, parent)
+        return model
+
+    def _cleanup(self, root):
+        super()._cleanup(root)
+        self.layout._cleanup(root)
+
+    def _update_layout(self, *events):
+        self.layout.param.set_param(**{event.name: event.new for event in events})
+
+    @param.depends("text", watch=True)
+    def _update_text(self):
+        if self.text_pane:
+            self.text_pane.object = self.text
+
+    @param.depends("value", watch=True)
+    def _update_value(self):
+        if self.progress:
+            self.progress.value = self.value
+
+    @param.depends("max", watch=True)
+    def _update_max(self):
+        if self.progress:
+            self.progress.max = self.max
+
+    def __call__(self, *args, **kwargs):
+        kwargs['indicator'] = self
+        if not self.write_to_console:
+            f = open(os.devnull, 'w')
+            kwargs['file'] = f
+        return ptqdm(*args, **kwargs)
+
+    __call__.__doc__ = ptqdm.__doc__
+
+    def pandas(self, *args, **kwargs):
+        kwargs['indicator'] = self
+        if not self.write_to_console and 'file' not in kwargs:
+            f = open(os.devnull, 'w')
+            kwargs['file'] = f
+        return ptqdm.pandas(*args, **kwargs)
+
+    def reset(self):
+        """Resets the parameters"""
+        self.value = self.param.value.default
+        self.text = self.param.text.default
