@@ -17,7 +17,7 @@ from bokeh.models.widgets import (
 
 from ..layout import Column, VSpacer
 from ..models import SingleSelect as _BkSingleSelect
-from ..util import as_unicode, isIn, indexOf, bokeh_version
+from ..util import as_unicode, isIn, indexOf
 from .base import Widget, CompositeWidget
 from .button import _ButtonBase, Button
 from .input import TextInput, TextAreaInput
@@ -130,6 +130,8 @@ class Select(SingleSelectBase):
         If set to 1 displays options as dropdown otherwise displays
         scrollable area.""")
 
+    _source_transforms = {'size': None}
+
     @property
     def _widget_type(self):
         return _BkSelect if self.size == 1 else _BkSingleSelect
@@ -205,9 +207,7 @@ class MultiChoice(_MultiSelectBase):
 
 
 _AutocompleteInput_rename = {'name': 'title', 'options': 'completions'}
-if bokeh_version < '2.3.0':
-    # disable restrict keyword
-    _AutocompleteInput_rename['restrict'] = None
+
 
 class AutocompleteInput(Widget):
 
